/*
 * Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Main line for coldread program.
 * 
 * Charles Shapiro 4 Jun 1999
 */
#include <stdio.h>  /* sscanf, everything. */
#include <getopt.h> /* getopt_long() */
#include <errno.h> /* strerror() */
#include <ctype.h> /* isspace, isprint */

// Define all variables (EXTERNAL = "extern" everywhere else)
#define EXTERNAL
#include "crstructs.h"
#include "coldread.tab.h"
#include "coldreader.h"

FILE * yyin; /* Input file for yyparse() routine. */
int yyleng;
int yylineno;
YYSTYPE yylval;
extern int yydebug;
int DebugLevel;

/*
 *  Usage message.
 */
void usage(const char *pname)
{
  fprintf(stderr,"Usage: %s  -p parmfile -i infile -o outfile [debug]\n\n",
  pname);
  fprintf(stderr,"Where parmfile is a coldread rule file. The infile\n");
  fprintf(stderr,"must contain cold read data to match the parmfile.\n");
  fprintf(stderr,"The outfile will contain a cold reading produced by this\n");
  fprintf(stderr,"program.\n");
  fprintf(stderr,"\nCharles Shapiro 25 Jul 1999\n\n");
}
/*
 * Print out an error message.
 */
void file_error_message(const char *errPreamble, const char *filename, 
int error_number)
{
  if(errPreamble)
    fprintf(stderr,"%s",errPreamble);

  fprintf(stderr," %s: %s\n",filename,strerror(error_number));

  return;
}
/*
 * Inform user of problem.
 */
void app_error_message(const char *message,const char *append)
{
  fprintf(stderr,"%s",message,append);
  if(append)
    fprintf(stderr,"[%s]\n",append);
  else
    fprintf(stderr,"\n");
}


/*
 * Retrieve values from input cold reading line.
 */
int parseKeyLine(char *lineToParse,char *outWordP,int *keyVal,int *keyBottom,
int *keyTop)
{
  char *testPtr;
  char *workPtr=lineToParse;
  int retVal=0;

  while(isspace(*workPtr))
    ++workPtr;
  testPtr=workPtr;
  while(! isspace(*workPtr))
  {
    *outWordP=*workPtr;
    ++outWordP;
    ++workPtr;
  }
  *outWordP='\0';

  /* Crude, but simple. Automatically scans over all whitespace. */
  sscanf(workPtr,"%d %d %d",keyTop,keyBottom,keyVal);

  if(workPtr == testPtr) /* Did we find a keyword? */
    retVal=-1;

  return retVal;
}
/*
 * Main line cold reader.
 */
int main(int argc, char *argv[])
{
  FILE *infile=0;
  FILE *outfile=0;
  FILE *parmsfile=0;
  int status=0;
  int foundOption = 0;
  int optionIndex=0;
  static struct option long_options[] = 
  {
    {"parmfile",1,0,'p' },
    {"infile",1,0,'i' },
    {"outfile",1,0,'o' },
    {"debug",1,0,'d' },
    {"stdin",0,0,'s'},
    {"stdout",0,0,'u'},
    {"calcscale",0,0,'c'},
    {0,0,0,0}
  };
  sentence *reading_sentences;
  int numSentences;


  optind=0; /* optind is defined in getopt.h */
  while(foundOption != -1)
  {
    foundOption = getopt_long(argc,argv,"d:i:o:p:suc",
    long_options,&optionIndex);
    switch(foundOption)
    {
    case 'i':
      infile=fopen(optarg,"r");
      if( ! infile)
      {
	file_error_message("Cannot read ",optarg,errno);
	status=-1;
      }
      break;
    case 'o':
      outfile=fopen(optarg,"w");
      if(! outfile) 
      {
	file_error_message("Cannot write",optarg,errno);
	status=-1;
      }
      break;
    case 'p':
      parmsfile=fopen(optarg,"r");
      if(! parmsfile)
      {
	file_error_message("Cannot read ",optarg,errno);
	status=-1;
      }
      break;
    case 'd':
      DebugLevel=atoi(optarg);
      break;
    case 's': /* Use stdin */
      infile=stdin;
      break;
    case 'u':
      outfile=stdout;
      break;
    case 'c':
      StoreScaleVal=1;
      break;
    case -1: // We are at the End.
      break;
    }
  }

  if(0 != status)
  {
    usage(argv[0]);
    exit(1);
  }
  if(0 == status)
  {
    if((! infile ) || (!outfile) || (! parmsfile))
    {
      fprintf(stderr,"Something did not open\n");
      status=-1;
    }
  }
  /*
  if(0 == status)      
    printf("Ready to Go! Debug level set to %d\n", DebugLevel);
  */
  if(0 == status)
  {
    initGlobalData();
    if(DebugLevel > 20)
      yydebug=1;
    else
      yydebug=0;
    yyin=parmsfile;
    yyparse();
  }

  if(0 == status)  
    status = do_reading(infile);

  if(0 == status)
  {
    calculate_expressions(Expression_pile);
    calculate_relationships(Relationship_pile);
    reading_sentences=(sentence *)calloc(1,sizeof(sentence));
    numSentences = calculate_reading(reading_sentences,Keyword_pile,
    Relationship_pile);
    sort_reading(&reading_sentences,numSentences);
    print_reading(reading_sentences,outfile); 
  }
  
  fclose(infile);
  fclose(outfile);
}
